--
--	haeckselklappe 
--	uses an animation, defined in the .i3d of the mod
--	to lift/lower the haeckselklappe
--
--	author:		fruktor
--	date:		09.06.2011
--	version:	0.1	- initial implementaion
--
--	Copyright (C) - www.eifok-team.de
--      _  __      _           _                       
--     (_)/ _|    | |         | |                      
--  ___ _| |_ ___ | | ________| |_ ___  __ _ _ __ ___  
-- / _ \ |  _/ _ \| |/ /______| __/ _ \/ _` | '_ ` _ \ 
--|  __/ | || (_) |   <       | ||  __/ (_| | | | | | |
-- \___|_|_| \___/|_|\_\       \__\___|\__,_|_| |_| |_|
--

Haeckselklappe = {};

function Haeckselklappe.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(AnimatedVehicle, specializations);
end;

function Haeckselklappe:load(xmlFile)

	self.updateTargetTimeCover = SpecializationUtil.callSpecializationsFunction("updateTargetTimeCover");

	self.haeckselklappe = {};
	self.haeckselklappe.index 		= Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.haeckselklappe#index"));
	self.haeckselklappe.charSet 	= getAnimCharacterSet(self.haeckselklappe.index);
	self.haeckselklappe.clip 		= getXMLString(xmlFile, "vehicle.haeckselklappe#clip");
	self.haeckselklappe.clipIdx 	= getAnimClipIndex(self.haeckselklappe.charSet, self.haeckselklappe.clip);	
	self.haeckselklappe.speedScale 	= getXMLFloat(xmlFile, "vehicle.haeckselklappe#speedScale");
	self.haeckselklappe.duration 	= getAnimClipDuration(self.haeckselklappe.charSet, self.haeckselklappe.clipIdx);
	self.haeckselklappe.curTime 	= getAnimTrackTime(self.haeckselklappe.charSet, 0);
	self.haeckselklappe.targetTime 	= 0;
	
	local x, y, z = getTranslation( self.fillAutoAimTargetNode );
	self.haeckselklappe.autoAimPos = {x, y, z};
	self.haeckselklappe.autoAimTrans = getXMLFloat(xmlFile, "vehicle.haeckselklappe#autoAimTrans");
	
end;

function Haeckselklappe:delete()
end;

function Haeckselklappe:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Haeckselklappe:keyEvent(unicode, sym, modifier, isDown)
end;

function Haeckselklappe:readStream(streamId, connection)
	--self.haeckselklappe.targetTime = streamReadFloat32(streamId);
	local curTime = streamReadFloat32(streamId);
	self.haeckselklappe.targetTime = curTime;
	--self:updateTargetTimeCover(curTime, true);
end;

function Haeckselklappe:writeStream(streamId, connection)
	streamWriteFloat32(streamId, self.haeckselklappe.targetTime);
end;

function Haeckselklappe:update(dt)
	
	local curTime = getAnimTrackTime(self.haeckselklappe.charSet, 0);
	self.haeckselklappe.curTime = curTime;

	if self:getIsActiveForInput() then	
		local targetTime = nil;
		if InputBinding.isPressed(InputBinding.Haeckselklappe_open) then
			targetTime = Utils.getMovedLimitedValues({self.haeckselklappe.curTime}, {0}, {self.haeckselklappe.duration}, 1, self.haeckselklappe.duration, dt, false);
		elseif InputBinding.isPressed(InputBinding.Haeckselklappe_close) then
			targetTime = Utils.getMovedLimitedValues({self.haeckselklappe.curTime}, {0}, {self.haeckselklappe.duration}, 1, self.haeckselklappe.duration, dt, true);
		end;
		if targetTime ~= nil then
			self:updateTargetTimeCover(targetTime[1]);
		end;
	end;
		
	self.jointmove = false;
	if math.abs(curTime - self.haeckselklappe.targetTime) > 5 then
		if curTime < self.haeckselklappe.targetTime then
			assignAnimTrackClip(self.haeckselklappe.charSet, 0, self.haeckselklappe.clipIdx, self.haeckselklappe.clip); 
			setAnimTrackSpeedScale(self.haeckselklappe.charSet, 0, self.haeckselklappe.speedScale);
			enableAnimTrack(self.haeckselklappe.charSet, 0);	
			self.jointmove = true;
		elseif curTime > self.haeckselklappe.targetTime then
			assignAnimTrackClip(self.haeckselklappe.charSet, 0, self.haeckselklappe.clipIdx, self.haeckselklappe.clip); 
			setAnimTrackSpeedScale(self.haeckselklappe.charSet, 0, -self.haeckselklappe.speedScale);
			enableAnimTrack(self.haeckselklappe.charSet, 0);	
			self.jointmove = true;
		end;
		local fac = self.haeckselklappe.targetTime/self.haeckselklappe.duration;
		setTranslation( self.fillAutoAimTargetNode, 
			self.haeckselklappe.autoAimPos[1], self.haeckselklappe.autoAimPos[2], self.haeckselklappe.autoAimPos[3]+fac*self.haeckselklappe.autoAimTrans );
	else
		--assignAnimTrackClip(self.haeckselklappe.charSet, 0, self.haeckselklappe.clipIdx, self.haeckselklappe.clip); 
		setAnimTrackSpeedScale(self.haeckselklappe.charSet, 0, 0);
		disableAnimTrack(self.haeckselklappe.charSet, 0);	
		clearAnimTrackClip(self.haeckselklappe.charSet, 0);		
		self.jointmove = false;
	end;
	
end;

function Haeckselklappe:updateTick(dt)
end;

function Haeckselklappe:draw()	

	g_currentMission:addExtraPrintText( string.format(g_i18n:getText("Haeckselklappe_CLOSEOPEN"), InputBinding.getKeyNamesOfDigitalAction(InputBinding.Haeckselklappe_close), InputBinding.getKeyNamesOfDigitalAction(InputBinding.Haeckselklappe_open) ) );
		
end;

function Haeckselklappe:updateTargetTimeCover(targetTime, noEventSend)
	UpdateTargetTimeCoverEvent1.sendEvent(self, targetTime, noEventSend);
	
	self.haeckselklappe.targetTime = targetTime;
	--print("self.haeckselklappe.targetTime="..self.haeckselklappe.targetTime);
end;

--
--
--
--
--
UpdateTargetTimeCoverEvent1 = {};
UpdateTargetTimeCoverEvent1_mt = Class(UpdateTargetTimeCoverEvent1, Event);

InitEventClass(UpdateTargetTimeCoverEvent1, "UpdateTargetTimeCoverEvent1");

function UpdateTargetTimeCoverEvent1:emptyNew()
    local self = Event:new(UpdateTargetTimeCoverEvent1_mt);
    self.className = "UpdateTargetTimeCoverEvent1";
    return self;
end;

function UpdateTargetTimeCoverEvent1:new(object, value)
    local self = UpdateTargetTimeCoverEvent1:emptyNew()
    self.object = object;
	self.value = value;
    return self;
end;

function UpdateTargetTimeCoverEvent1:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.object = networkGetObject(id);
	self.value = streamReadFloat32(streamId);
    self:run(connection);
end;

function UpdateTargetTimeCoverEvent1:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteFloat32(streamId, self.value);
end;

function UpdateTargetTimeCoverEvent1:run(connection)
	self.object:updateTargetTimeCover(self.value, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(UpdateTargetTimeCoverEvent1:new(self.object, self.value), nil, connection, self.object);
	end;
end;

function UpdateTargetTimeCoverEvent1.sendEvent(vehicle, value, noEventSend)

	if vehicle.haeckselklappe.targetTime ~= value then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(UpdateTargetTimeCoverEvent1:new(vehicle, value), nil, nil, vehicle);
			else
				g_client:getServerConnection():sendEvent(UpdateTargetTimeCoverEvent1:new(vehicle, value));
			end;
		end;
	end;
end;